<?php


namespace App\Services\pingxx;

use Api\PhpUtils\Log\FileLog;
use Pingpp\Order;
use Pingpp\Pingpp;
use Pingpp\OrderRefund;
use Helpers\Logger;
use Api\PhpUtils\Common\IP;


class PingxxService
{
    private static $instance = null;

    private function __construct()
    {
        Pingpp::setApiKey('sk_test_KCaD0O5KyjzPinLynHzH4Cu1'); // 设置 API Key
        Pingpp::setPrivateKeyPath(ROOT_PATH . '/conf/your_rsa_private_key.pem'); // 设置私钥
        Pingpp::setAppId('app_9m1ubDG4e1mPXLCG'); // 设置 App ID
    }

    private function __clone() {}

    public static function getInstance() {
        if(empty(self::$instance)) {
            self::$instance = new self;
        }

        return self::$instance;
    }


    /**
     * @param $order_item_id
     * @param $user_id
     * @throws RefundException
     * 发起退款请求
     */
    public function sendRefund($refundData) {

        try {

            FileLog::info('ping++:' . '退款订单发起', json_encode($refundData));
            $royalty_users = [];

            $ret = OrderRefund::create(
                $refundData['charge'],
                [
                    'charge' => $refundData['charge'],  //在该订单中，需要退款的 charge 对象 ID，默认全部退款。
                    'charge_amount' => $refundData['charge_amount'],    //charge 退款金额，默认为全额退款。必须和 charge 参数同时使用。单位：分。
                    'description' => '用户申请退款',    //退款附加说明。
                    'metadata' => $refundData['metadata'],
                    'refund_mode' => 'to_source',              //退款模式。原路退回：to_source，退至余额：to_balance。默认为原路返回。
                ]
            );
            FileLog::info('ping++:' . '退款订单返回', json_encode(['res'=>$ret]));
            $response = json_decode($ret, true);
            return $response;

        } catch (\Pingpp\Error\Base $e) {

            if ($e->getHttpStatus() != null) {
                $msg = $e->getHttpBody();
            } else {
                $msg = $e->getMessage();
            }

            exit($msg);
            throw new \Exception($msg, '2001');
        }
    }

    /**
     * @param $orderId ping++ 支付订单ID
     * @param $refundId ping++ 退款ID
     * @return mixed
     * 退款信息查询
     */
    public function getRefund($orderId, $refundId) {
        try {
            FileLog::info('ping++:' . '退款查询发起', json_encode(['orderId'=>$orderId, 'refundId'=>$refundId]));
            $ret = OrderRefund::retrieve($orderId, $refundId);
            FileLog::info('ping++:' . '退款查询返回', json_encode(['res'=>$ret]));
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            if ($e->getHttpStatus() != null) {
                header('Status: ' . $e->getHttpStatus());
                echo $e->getHttpBody();
            } else {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $order
     * @return mixed
     * 创建订单
     */
    public function createOrder($order, $metadata) {
        try {

            FileLog::info('ping++:' . '创建订单发起', json_encode($order));
            $royalty_users = [];
            $ret = Order::create(
                [
                    "amount" => $order['pay_amount'],
                    "app" => Pingpp::getAppId(),
                    "merchant_order_no" => $order['pay_order_id'],
                    "subject" => "用户购买",
                    "currency" => "cny",
                    "body" => "购买商品",
                    "uid" => $order['user_id'],
                    "client_ip" => IP::ip(),
                    'receipt_app' => Pingpp::getAppId(),    // 收款方应用
                    'service_app' => Pingpp::getAppId(),    // 服务方应用
                    'metadata' => $metadata,    // 商户定制信息
                ]
            );

            FileLog::info('ping++:' . '创建订单返回', json_encode(['res'=>$ret]));
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            // 捕获报错信息
            if ($e->getHttpStatus() != null) {
                echo $e->getHttpStatus() . PHP_EOL;
                echo $e->getHttpBody() . PHP_EOL;
            } else {
                echo $e->getMessage() . PHP_EOL;
            }
        }
    }

    /**
     * @param $orderId
     * 订单查询
     */
    public function getOrder($orderId) {

        try {
            FileLog::info('ping++:' . '查询订单发起', json_encode(['orderId'=>$orderId]));
            $ret = Order::retrieve($orderId);

            FileLog::info('ping++:' . '查询订单返回', json_encode(['res'=>$ret]));
            $respose = json_decode($ret, true);
            return $respose;

        } catch (\Pingpp\Error\Base $e) {
            if ($e->getHttpStatus() != null) {
                header('Status: ' . $e->getHttpStatus());
                echo $e->getHttpBody();
            } else {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $raw
     * @param $header
     * @return int
     * 校验回调是否合法
     */
    public function verifySignature($raw, $headers) {

        $signature = isset($headers['X-Pingplusplus-Signature']) ? $headers['X-Pingplusplus-Signature'] : null;

        $pub_key_path = ROOT_PATH . '/conf/pingpp_rsa_public_key.pem';
        $pub_key_path = ROOT_PATH . '/vendor/pingplusplus/pingpp-php/example/pingpp_rsa_public_key.pem';
        $pub_key_contents = file_get_contents($pub_key_path);
        return openssl_verify($raw, base64_decode($signature), $pub_key_contents, 'sha256');
    }
}